package com.yunfeiyun.agmp.fms.service;

import com.yunfeiyun.agmp.common.core.domain.AjaxResult;
import com.yunfeiyun.agmp.fms.domain.TssSourceinfo;
import com.yunfeiyun.agmp.fms.domain.reqvo.AddTssSourceinfoReqVo;
import com.yunfeiyun.agmp.fms.domain.resvo.TssSourceinfoListResVo;

import java.util.List;

/**
 * 溯源信息Service接口
 * 
 * @author 杨晓辉
 * @date 2023-06-25
 */
public interface ITssSourceinfoService 
{
    /**
     * 查询溯源信息
     * 
     * @param sourceinfoId 溯源信息主键
     * @return 溯源信息
     */
    public TssSourceinfo selectTssSourceinfoBySourceinfoId(String sourceinfoId);

    /**
     * 查询溯源信息列表
     * 
     * @param tssSourceinfo 溯源信息
     * @return 溯源信息集合
     */
    public List<TssSourceinfoListResVo> selectTssSourceinfoList(TssSourceinfo tssSourceinfo);

    /**
     * 新增溯源信息
     * 
     * @param tssSourceinfo 溯源信息
     * @return 结果
     */
    public int insertTssSourceinfo(TssSourceinfo tssSourceinfo);
    public AjaxResult insertTssSourceinfo(AddTssSourceinfoReqVo reqVo);

    /**
     * 修改溯源信息
     * 
     * @param tssSourceinfo 溯源信息
     * @return 结果
     */
    public int updateTssSourceinfo(TssSourceinfo tssSourceinfo);
    public AjaxResult updateTssSourceinfo(AddTssSourceinfoReqVo reqVo);

    /**
     * 批量删除溯源信息
     * 
     * @param sourceinfoIds 需要删除的溯源信息主键集合
     * @return 结果
     */
    public int deleteTssSourceinfoBySourceinfoIds(String[] sourceinfoIds);

    /**
     * 删除溯源信息信息
     * 
     * @param sourceinfoId 溯源信息主键
     * @return 结果
     */
    public int deleteTssSourceinfoBySourceinfoId(String sourceinfoId);
}
